
#include <string.h>
#include "LView.h"
#include "Logger.h"

LogEntry::LogEntry( const char* asource, int severity, const char* atext)
	: prev(this), next(this), sev(severity % 4)
{
	int tsize = ::strlen( asource);
	
	source = new char [ tsize + 1];
	::strcpy( source, asource);
	
	tsize = ::strlen( atext);
	
	text = new char [ tsize + 1];
	::strcpy( text, atext);

}


LogEntry::~LogEntry()
{
	delete [] source;
	delete [] text; 
}

LView::LView(BRect rect, char *name)
	: BView(rect, name, B_FOLLOW_ALL, B_WILL_DRAW),
	  my_lineheight(0),
	  my_head( "Head", 0, "Head"),
	  my_frozen_at(0),
	  my_display_count(0),
	  my_filter( LOGGER_INFORMATIONAL)
{
	// Add some entries to test things
	AddToLog( "Logger", LOGGER_INFORMATIONAL, "Logging started");
}


LView::~LView()
{
	LogEntry* here = my_head.next;
	
	while( here != &my_head)
	{
		LogEntry* next_one = here->next;
		delete here;
		here = next_one;
	}
}

void LView::AttachedToWindow()
{
	font_info info;

	SetFontName("Kate");
	SetFontSize(9);
	GetFontInfo( &info);
	
	my_lineheight = info.ascent + info.descent + info.leading;
	my_textoffset = info.ascent;
}

void LView::AddToLog( const char* source, int severity, const char* m)
{
	LogEntry* newentry = new LogEntry( source, severity, m);
	LogEntry* lastone = my_head.prev;
	
	newentry->prev = lastone;
	newentry->next = &my_head;
	lastone->next = newentry;
	my_head.prev = newentry;
	
	float pos = my_lineheight * my_display_count;

	if ( severity <= my_filter && !my_frozen_at)
		my_display_count++;
	
	
	BRect  invalid_rect(
		0, pos,
		10000, pos + my_lineheight
	);
	
	Invalidate( invalid_rect);
}

float LView::DataHeight()
{
	return DisplayedCount() * my_lineheight;
}

float LView::DataWidth()
{
	return 1000;
}

void LView::CalcDisplayCount()
{
	int display_count = 0;
	
	for( LogEntry* here = my_head.next; 
	     (here != &my_head) && (here->prev != my_frozen_at);
	     here = here->next)
	{
		if ( here->sev <= my_filter)
		{
			display_count++;
		}		
	}

	my_display_count = display_count;
}

void LView::ClearLog()
{
	LogEntry* here = my_head.next;
	
	while( here != &my_head)
	{
		LogEntry* next_one = here->next;
		delete here;
		here = next_one;
	}
	
	my_head.next = my_head.prev = &my_head;
	my_display_count = 0;

	if ( my_frozen_at)
		my_frozen_at = &my_head;

	Invalidate();
}

void LView::Freeze( bool yes)
{
	if ( yes)
	{
		my_frozen_at = my_head.prev;
	}
	else
	{
		my_frozen_at = 0;
		CalcDisplayCount();
		Invalidate();
	}	
}

void LView::Filter( LoggerSeverity sev)
{
	my_filter = sev;
	CalcDisplayCount();
	Invalidate();
}

const rgb_color colors[] = {
 //     RED   GREEN	   BLUE	 ALPHA
	{	255,	0,		0,	  128 },
	{	255,    255,    0,    128 },
	{   0,      0,     255,   128 },
	{   0,      255,    0,    128 }
}; 

void LView::Draw(BRect updaterect)
{
	LogEntry* here;
	
	const int LEFTMARG = 3;
	const int RECTBORDER = 1;
	const int AFTERINDICATOR = 5;

	rgb_color current_high = HighColor();

	BPoint textpos( 
		LEFTMARG + my_lineheight + AFTERINDICATOR,
		my_textoffset
	);

	BRect  indicator_rect(
		LEFTMARG + RECTBORDER,
		RECTBORDER,
		LEFTMARG + my_lineheight - RECTBORDER,
		my_lineheight - RECTBORDER
	);
	
	for( here = my_head.next; 
	     (here != &my_head) && (here->prev != my_frozen_at);
	     here = here->next)
	{
		// printf( "Expose %f,%f, Indicator %f,%f\n",
		//        updaterect.top, updaterect.bottom,
		//        indicator_rect.top, indicator_rect.bottom 
		//      );
		
		if ( here->sev <= my_filter)
		{
			if ( (updaterect.top <= indicator_rect.bottom)
		   		&& (updaterect.bottom >= indicator_rect.top))
			{
				// Draw indicator
				SetHighColor( colors[here->sev]);
				FillRect( indicator_rect);
				SetHighColor(current_high);
				StrokeRect( indicator_rect);
		
				// Draw text
				DrawString( here->source, textpos);
				DrawString( ": ");
				DrawString( here->text);
			}
			textpos.y += my_lineheight;
			indicator_rect.top += my_lineheight;
			indicator_rect.bottom += my_lineheight;
		}		
	}
}

