#include "Logger.h"
#include <Application.h>
#include "LWindow.h"

const int MESSAGE_CLASS_MASK = 0xf00;

enum MESSAGE_CLASSES {
	MC_EXTERNAL = 256,
	MC_MENU     = 512
};

enum MenuConstants {
	MENU_OPEN = MC_MENU,
	MENU_SAVE,
	MENU_SAVEAS,
	MENU_FREEZE,
	MENU_DISPFILTER_C,
	MENU_DISPFILTER_S,
	MENU_DISPFILTER_W,
	MENU_DISPFILTER_I,
	MENU_CLEAR,
	MENU_DISCARD,
	MENU_LOGFILTER_C,
	MENU_LOGFILTER_S,
	MENU_LOGFILTER_W,
	MENU_LOGFILTER_I,
	MENU_CAPACITY,
	MENU_GENERAL_HELP
};

LWindow::LWindow(BRect frame)
	: 	myview(0),
	    my_vscrollbar(0),
	    my_discarding(FALSE),
	    my_filter( LOGGER_INFORMATIONAL),
		BWindow(frame, "Logger", B_DOCUMENT_WINDOW,0) 
{
	BRect clientrect( 0, B_MENU_BAR_HEIGHT + 1, 
	                  frame.Width() - B_V_SCROLL_BAR_WIDTH, 
	                  frame.Height() - B_H_SCROLL_BAR_HEIGHT
	                );
	
	BRect hscrollrect( 0, clientrect.bottom + 1,
	                   clientrect.right, frame.Height()
	                 );
	
	BRect vscrollrect( clientrect.right + 1, B_MENU_BAR_HEIGHT,
	                   frame.Width(), clientrect.bottom
	                 );
	
	SetupMenu();
	myview = new LView( clientrect, "Client");
	AddChild( myview);
	AddChild(
		new BScrollBar( hscrollrect, "hscroll", myview, 
		                0, 1000, B_HORIZONTAL)
	);
	AddChild(
		my_vscrollbar = new BScrollBar( vscrollrect, "vscroll", myview, 
		                0, 1000, B_VERTICAL)
	);
	
	ReCalcScrollBars();
}

void LWindow::ReCalcScrollBars()
{
	if ( my_vscrollbar)
	{
		Lock();
		float dataheight = myview->DataHeight();
		float viewheight = myview->Bounds().Height();
	
		if ( dataheight < viewheight)
		{
			// We cannot scroll
			my_vscrollbar->SetRange( 0,0);
			my_vscrollbar->SetProportion(1.0);
		}
		else
		{
			// Scrolling is possible, adjust stuff, but if already
			// scrolled to the max, then try to make the scrollbar
			// "sticky"
			
			long oldmax;
			long oldmin;
			long oldvalue = my_vscrollbar->Value();
			long newmax = dataheight - viewheight;
			
			my_vscrollbar->GetRange( &oldmin, &oldmax);
			
			my_vscrollbar->SetRange( 0, newmax);
			my_vscrollbar->SetProportion(viewheight/dataheight);
			
			if ( oldvalue == oldmax)
			{
				my_vscrollbar->SetValue( newmax);	
			}
		}
		Unlock();
	}
}

void LWindow::FrameResized( float /* width */, float /* height */)
{
	ReCalcScrollBars();
}

void LWindow::SetupMenu()
{
	BMenuItem* item;	

	BRect menu_rect( 0.0, 0.0, Bounds().right+1, B_MENU_BAR_HEIGHT);
	
	
	BMenuBar* mainmenu = new BMenuBar(
							menu_rect,
							"",
							B_FOLLOW_LEFT_RIGHT | B_FOLLOW_TOP,
							B_ITEMS_IN_ROW,
							FALSE	// Don't resize to fit items
	                     );
/*	
	BMenu* filemenu = new BMenu("File");
	filemenu->AddItem(new BMenuItem( "Open...", new BMessage( MENU_OPEN), 'O'));
	filemenu->AddItem(new BMenuItem( "Save", new BMessage( MENU_SAVE), 'S'));
	filemenu->AddItem(new BMenuItem( "Save as...", new BMessage( MENU_SAVEAS)));
*/
	BMenu* dispfilt = new BMenu("Filter level");
	dispfilt->SetRadioMode( TRUE);
	dispfilt->AddItem( new BMenuItem( "Critical only", new BMessage( MENU_DISPFILTER_C)));
	dispfilt->AddItem( new BMenuItem( "Critical and severe", new BMessage( MENU_DISPFILTER_S)));
	dispfilt->AddItem( new BMenuItem( "All but informational", new BMessage( MENU_DISPFILTER_W)));
	item = new BMenuItem( "All", new BMessage( MENU_DISPFILTER_I));
	item->SetMarked( TRUE);
	dispfilt->AddItem( item);

	BMenu* logfilt = new BMenu("Filter level");
	logfilt->SetRadioMode( TRUE);
    logfilt->AddItem( new BMenuItem( "Critical only", new BMessage( MENU_LOGFILTER_C)));
	logfilt->AddItem( new BMenuItem( "Critical and severe", new BMessage( MENU_LOGFILTER_S)));
	logfilt->AddItem( new BMenuItem( "All but informational", new BMessage( MENU_LOGFILTER_W)));
	item = new BMenuItem( "All", new BMessage( MENU_LOGFILTER_I));
	item->SetMarked( TRUE);
	logfilt->AddItem( item);
	

	BMenu* displaymenu = new BMenu("Display");
	displaymenu->AddItem(new BMenuItem( "Freeze", new BMessage( MENU_FREEZE), 'F'));
	displaymenu->AddItem( dispfilt);

	BMenu* logmenu = new BMenu("Log");
	logmenu->AddItem(new BMenuItem( "Clear", new BMessage( MENU_CLEAR), 'C'));
	logmenu->AddItem(new BMenuItem( "Discard", new BMessage( MENU_DISCARD)));
	logmenu->AddItem( logfilt);
	logmenu->AddItem(new BMenuItem( "Capacity...", new BMessage( MENU_CAPACITY)));

/*
	BMenu* helpmenu = new BMenu("Help");
	helpmenu->AddItem(new BMenuItem( "General help", new BMessage( MENU_GENERAL_HELP), 'H'));
*/	

//	mainmenu->AddItem(filemenu);
	mainmenu->AddItem(logmenu);
	mainmenu->AddItem(displaymenu);
//	mainmenu->AddItem(helpmenu);
	
	AddChild(mainmenu);
}

void LWindow::MessageReceived( BMessage* message)
{
	if ( message)
	{
		switch( message->what & MESSAGE_CLASS_MASK)
		{
			case MC_EXTERNAL:
				if ( message->what == LOGGER_MESSAGE)
				{
					const char *src = message->FindString( LOGGER_SOURCEKEY);
					const char *txt = message->FindString( LOGGER_TEXTKEY);
					int  sev = int(message->FindLong( LOGGER_SEVERITYKEY));
					
					// Check validity and filter it
					if ( (sev >=0 ) && src && txt && (sev <= my_filter))
					{
						myview->AddToLog( src, sev, txt); 
						ReCalcScrollBars();
					}
				}
				break;
			
			case MC_MENU:
				ProcessMenuPick( message);
				break;
												
			default:
				break;
		}
	}	
}


void LWindow::ProcessMenuPick( BMessage* message)
{
	BMenuItem* item = (BMenuItem*)message->FindObject("source");

	if ( item && item->IsEnabled())
	{
		bool marked = item->IsMarked();
		
		switch( message->what)
		{
			case MENU_CLEAR:
				myview->ClearLog();
				break;
				
			case MENU_FREEZE:
				myview->Freeze( !marked);
				item->SetMarked( !marked);
				break;
				
			case MENU_DISCARD:
			    my_discarding = !marked;
				item->SetMarked( !marked);
				break;
				
			case MENU_LOGFILTER_C:
			case MENU_LOGFILTER_S:
			case MENU_LOGFILTER_W:
			case MENU_LOGFILTER_I:
				my_filter = LoggerSeverity(message->what - MENU_LOGFILTER_C);
				break;
	
			case MENU_DISPFILTER_C:
			case MENU_DISPFILTER_S:
			case MENU_DISPFILTER_W:
			case MENU_DISPFILTER_I:
				myview->Filter( LoggerSeverity(message->what - MENU_DISPFILTER_C));
				break;

//			case MENU_GENERAL_HELP:
//			case MENU_OPEN:
//			case MENU_SAVE:
//			case MENU_SAVEAS:
//			case MENU_CAPACITY:
			default:
				myview->AddToLog( "Logger", LOGGER_WARNING, "Unimplemented menu option");
				break;
		}
		ReCalcScrollBars();
	}
}

bool LWindow::QuitRequested()
{
	be_app->PostMessage(B_QUIT_REQUESTED);
	return(TRUE);
}
